#if !defined(__GHCPLATFORM_H__)
#define __GHCPLATFORM_H__

#define GHC_STAGE (1 + 1)

#define BuildPlatform_TYPE  i386_unknown_linux
#define HostPlatform_TYPE   i386_unknown_linux

#define i386_unknown_linux_BUILD  1
#define i386_unknown_linux_HOST  1

#define i386_BUILD_ARCH  1
#define i386_HOST_ARCH  1
#define BUILD_ARCH  "i386"
#define HOST_ARCH  "i386"

#define linux_BUILD_OS  1
#define linux_HOST_OS  1
#define BUILD_OS  "linux"
#define HOST_OS  "linux"

#define unknown_BUILD_VENDOR  1
#define unknown_HOST_VENDOR  1
#define BUILD_VENDOR  "unknown"
#define HOST_VENDOR  "unknown"


#endif /* __GHCPLATFORM_H__ */
