/* ----------------------------------------------------------------------------
   (c) The University of Glasgow 2004

   Flags used in runProcess.c and for System.Process.Internals
   ------------------------------------------------------------------------- */

#define RUN_PROCESS_IN_CLOSE_FDS 0x1
#define RUN_PROCESS_IN_NEW_GROUP 0x2
#define RUN_PROCESS_DETACHED     0x4
#define RUN_PROCESS_NEW_SESSION  0x8
#define RUN_PROCESS_NEW_CONSOLE  0x10
